@@ -13,4 +13,4 @@ state.add_interpolated([0, 1], 1)
 
 eq = DiffusionPDE()  # define the pde
 result = eq.solve(state, t_range=1, dt=0.005)
-result.plot(show=True)
+result.plot()
