@@ -98,8 +98,8 @@ class FieldCollection(FieldBase):
                 field._data_flat = self.data[self._slices[i]]
                 
                 # check whether the field data is based on our data field
-                assert field.data.base is self.data
                 assert field.data.shape == field_shape
+                assert field.data.base is self.data
          
          
     def __repr__(self):
@@ -326,7 +326,7 @@ class FieldCollection(FieldBase):
         # if data is None, the data of the individual fields is copied in their
         # copy() method above. The underlying data is therefore independent from
         # the current field 
-        return self.__class__(fields, data=data, label=label)
+        return self.__class__(fields, data=data, label=label, copy_fields=False)
 
 
     def interpolate_to_grid(self, grid: GridBase,
@@ -441,22 +441,20 @@ class FieldCollection(FieldBase):
         
     @plot_on_figure(update_method='_update_plot')
     def plot(self,
-             fig,
              kind: str = 'auto',
              resize_fig: bool = True,
+             fig=None,
              **kwargs) -> List[PlotReference]:
         r""" visualize all the fields in the collection
         
         Args:
-            fig (:class:`matplotlib.figures.Figure`):
-                Figure that is used for plotting. If omitted, a new figure is
-                created.
             kind (str):
                 Determines the visualizations. Supported values are `image`, 
                 `line`, `vector`, or `interactive`. Alternatively, `auto`
                 determines the best visualization based on each field itself.
             resize_fig (bool):
-                Whether to resize the figure
+                Whether to resize the figure to adjust to the number of panels
+            {PLOT_ARGS}
             \**kwargs:
                 All additional keyword arguments are forwarded to the actual
                 plotting function.
@@ -472,7 +470,7 @@ class FieldCollection(FieldBase):
         axs = fig.subplots(1, len(self))
         
         # plot all the elements into the respective axes
-        reference = [field.plot(kind=kind, ax=ax, show=False, **kwargs)
+        reference = [field.plot(kind=kind, ax=ax, action='create', **kwargs)
                      for field, ax in zip(self.fields, axs)]
                 
         # return the references for all subplots
