@@ -26,6 +26,7 @@ from ...tools.docstrings import fill_in_docstring
 
 
 
+@PolarGrid.register_operator('laplace', rank_in=0, rank_out=0)
 @fill_in_docstring
 def make_laplace(bcs: Boundaries) -> Callable:
     """ make a discretized laplace operator for a polar grid
@@ -79,6 +80,7 @@ def make_laplace(bcs: Boundaries) -> Callable:
 
 
 
+@PolarGrid.register_operator('gradient', rank_in=0, rank_out=1)
 @fill_in_docstring
 def make_gradient(bcs: Boundaries) -> Callable:
     """ make a discretized gradient operator for a polar grid
@@ -133,6 +135,7 @@ def make_gradient(bcs: Boundaries) -> Callable:
 
 
 
+@PolarGrid.register_operator('divergence', rank_in=1, rank_out=0)
 @fill_in_docstring
 def make_divergence(bcs: Boundaries) -> Callable:
     """ make a discretized divergence operator for a polar grid
@@ -205,6 +208,7 @@ def make_divergence(bcs: Boundaries) -> Callable:
 
     
     
+@PolarGrid.register_operator('vector_gradient', rank_in=1, rank_out=2)
 @fill_in_docstring
 def make_vector_gradient(bcs: Boundaries) -> Callable:
     """ make a discretized vector gradient operator for a polar grid
@@ -235,6 +239,7 @@ def make_vector_gradient(bcs: Boundaries) -> Callable:
 
 
 
+@PolarGrid.register_operator('tensor_divergence', rank_in=2, rank_out=1)
 @fill_in_docstring
 def make_tensor_divergence(bcs: Boundaries) -> Callable:
     """ make a discretized tensor divergence operator for a polar grid
@@ -322,6 +327,7 @@ def _get_laplace_matrix(bcs):
 
 
 
+@PolarGrid.register_operator('poisson_solver', rank_in=0, rank_out=0)
 @fill_in_docstring
 def make_poisson_solver(bcs: Boundaries, method: str = 'auto') -> Callable:
     """ make a operator that solves Poisson's equation
@@ -337,18 +343,3 @@ def make_poisson_solver(bcs: Boundaries, method: str = 'auto') -> Callable:
     """
     matrix, vector = _get_laplace_matrix(bcs)
     return make_general_poisson_solver(matrix, vector, method)
-
-       
-# register all operators with the grid class
-PolarGrid.register_operator('laplace', make_laplace,
-                            rank_in=0, rank_out=0)
-PolarGrid.register_operator('gradient', make_gradient,
-                            rank_in=0, rank_out=1)
-PolarGrid.register_operator('divergence', make_divergence,
-                            rank_in=1, rank_out=0)
-PolarGrid.register_operator('vector_gradient', make_vector_gradient,
-                            rank_in=1, rank_out=2)
-PolarGrid.register_operator('tensor_divergence', make_tensor_divergence,
-                            rank_in=2, rank_out=1)
-PolarGrid.register_operator('poisson_solver', make_poisson_solver,
-                            rank_in=0, rank_out=0)
