@@ -4,6 +4,7 @@ Solvers define how a pde is solved, i.e., advanced in time.
 .. autosummary::
    :nosignatures:
 
+   ~controller.Controller
    ~explicit.ExplicitSolver
    ~implicit.ImplicitSolver
    ~scipy.ScipySolver
@@ -14,6 +15,7 @@ Solvers define how a pde is solved, i.e., advanced in time.
 
 from typing import List
 
+from .controller import Controller
 from .explicit import ExplicitSolver
 from .implicit import ImplicitSolver
 from .scipy import ScipySolver
@@ -31,5 +33,5 @@ def registered_solvers() -> List[str]:
 
 
 
-__all__ = ['ExplicitSolver', 'ImplicitSolver', 'ScipySolver',
+__all__ = ['Controller', 'ExplicitSolver', 'ImplicitSolver', 'ScipySolver',
            'registered_solvers']
