@@ -4,7 +4,7 @@ Defines a class storing data in memory.
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de> 
 """
 
-from typing import Optional, List, Sequence
+from typing import Optional, Sequence, Union, Tuple, List  # @UnusedImport
 from contextlib import contextmanager
 
 import numpy as np
@@ -16,19 +16,23 @@ from ..fields.base import FieldBase
     
     
 class MemoryStorage(StorageBase):
-    """ simple storage in memory """
+    """ store discretized fields in memory """
     
     
-    def __init__(self, times: Optional[List[float]] = None,
-                 fields: Optional[Sequence[FieldBase]] = None,
+    def __init__(self, times: Optional[Sequence[float]] = None,
+                 data: Optional[List[np.ndarray]] = None,
+                 field_obj: FieldBase = None,
                  info: InfoDict = None,
                  write_mode: str = 'truncate_once'):
         """
         Args:
             times (:class:`numpy.ndarray`):
                 Sequence of times for which data is known
-            fields (list of :class:`~pde.fields.FieldBase`):
+            data (list of :class:`~numpy.ndarray`):
                 The field data at the given times
+            field_obj (:class:`~pde.fields.base.FieldBase`):
+                An instance of the field class that can store data for a single
+                time point.
             info (dict):
                 Supplies extra information that is stored in the storage
             write_mode (str):
@@ -40,18 +44,12 @@ class MemoryStorage(StorageBase):
                 'readonly' will disable writing completely.
         """
         super().__init__(info=info, write_mode=write_mode)
-        self.times: List[float] = [] if times is None else times
-        
-        if fields is None:
-            self._grid = None
-            self.data = []
-        else:
-            self._grid = fields[0].grid
-            self.data = [fields[0].data]
-            for field in fields[1:]:
-                if self._grid != field.grid:
-                    raise ValueError('Grids of the fields are incompatible')
-                self.data.append(field.data)
+        self.times: List[float] = [] if times is None else list(times)
+        if field_obj is not None:
+            self._field = field_obj.copy()
+            self._grid = field_obj.grid
+        self.data: List[np.ndarray] = [] if data is None else data
+        if len(self.data) > 0:
             self._data_shape = self.data[0].shape
             
         # check consistency
@@ -60,12 +58,53 @@ class MemoryStorage(StorageBase):
                              f'inconsistent ({len(self.times)} != '
                              f'{len(self.data)}).')
             
+            
+    @classmethod
+    def from_fields(cls, times: Optional[Sequence[float]] = None,
+                    fields: Optional[Sequence[FieldBase]] = None,
+                    info: InfoDict = None,
+                    write_mode: str = 'truncate_once') -> "MemoryStorage":
+        """ create MemoryStorage from a list of fields
+        
+        Args:
+            times (:class:`numpy.ndarray`):
+                Sequence of times for which data is known
+            fields (list of :class:`~pde.fields.FieldBase`):
+                The fields at all given time points
+            info (dict):
+                Supplies extra information that is stored in the storage
+            write_mode (str):
+                Determines how new data is added to already existing data.
+                Possible values are: 'append' (data is always appended),
+                'truncate' (data is cleared every time this storage is used
+                for writing), or 'truncate_once' (data is cleared for the first
+                writing, but appended subsequently). Alternatively, specifying
+                'readonly' will disable writing completely.
+        """
+        if fields is None:
+            field_obj = None
+            data = None
+        else:
+            field_obj = fields[0]
+            data = [fields[0].data]
+            for field in fields[1:]:
+                if field_obj.grid != field.grid:
+                    raise ValueError('Grids of the fields are incompatible')
+                data.append(field.data)
+            
+        return cls(times, data=data, field_obj=field_obj, info=info,
+                   write_mode=write_mode)
+    
 
     @classmethod            
     def from_collection(cls, storages: Sequence["StorageBase"],
                         label: str = None) -> "MemoryStorage":
         """ combine multiple memory storages into one
         
+        This method can be used to combine multiple time series of different
+        fields into a single representation. This requires that all time series
+        contain data at the same time points.
+        
         Args:
             storages (list): 
                 A collection of instances of
@@ -98,7 +137,7 @@ class MemoryStorage(StorageBase):
         fields = [FieldCollection(d, label=label)  # type: ignore
                   for d in data]
         
-        return cls(times, fields=fields)  # type: ignore
+        return cls.from_fields(times, fields=fields)
             
         
     def clear(self, clear_data_shape: bool = False) -> None:
@@ -164,7 +203,16 @@ class MemoryStorage(StorageBase):
 
 @contextmanager
 def get_memory_storage(field: FieldBase, info: InfoDict = None):
-    """ initialize the storage for writing data
+    """ a context manager that can be used to create a MemoryStorage
+    
+    Example:
+        This can be used to quickly store data::
+        
+            with get_memory_storage(field_class) as storage:
+                storage.append(numpy_array0, 0)
+                storage.append(numpy_array1, 1)
+                
+            # use storage thereafter
     
     Args:
         field (:class:`~pde.fields.FieldBase`):
@@ -172,6 +220,9 @@ def get_memory_storage(field: FieldBase, info: InfoDict = None):
             and the data_shape
         info (dict):
             Supplies extra information that is stored in the storage
+            
+    Yields:
+        :class:`MemoryStorage`
     """
     storage = MemoryStorage()
     storage.start_writing(field, info)
