@@ -18,21 +18,8 @@ Tools for plotting and controlling plot output using context managers
 '''
 
 
-# Remarks
-# 
-# * Plotting use cases:
-#   - plot directly with show=True -> show window
-#       - default mode for outermost plot call
-#   - plot to file with show=False -> destroy window after saving
-#   - create plot to keep for later manipulation with show=False
-#   - create and show plot, but capture in ipython output
-# * These behaviors should be controlled by a single flag
-#   - potential names: behavior, action
-# * Plot wrapper must keep track of whether they handle the window 
-
-
-import re
 import contextlib
+import functools
 import logging
 import warnings
 from typing import Type, Dict, Tuple, Any, TYPE_CHECKING  # @UnusedImport
@@ -41,34 +28,42 @@ import matplotlib as mpl
 import matplotlib.pyplot as plt
 import matplotlib.axes as mpl_axes
 
+from ..tools.docstrings import replace_in_docstring
+
 if TYPE_CHECKING:
     from ..grids.base import GridBase  # @UnusedImport
 
  
- 
-_IS_PLOTTING = False  # global flag to detect nested plotting calls
 
-
-
-@contextlib.contextmanager
-def nested_plotting_check():
+class nested_plotting_check():
     """ context manager that checks whether it is the root plotting call
     
-    Yields:
-        bool: The context manager yields True for the first usage and False for
-        all nested calls.
-    """
-    global _IS_PLOTTING
-
-    if _IS_PLOTTING:
-        # this is a nested plot call
-        yield False
+    Example:
+        The context manager can be used in plotting calls to check for nested
+        plotting calls::
         
-    else:
-        # this is the outermost plot call
-        _IS_PLOTTING = True
-        yield True
-        _IS_PLOTTING = False
+            with nested_plotting_check() as is_outermost_plot_call:
+                make_plot(...)  # could potentially call other plotting methods
+                if is_outermost_plot_call:
+                    plt.show()
+    
+    """
+    
+    _is_plotting = False  # class variable keeping track of nesting
+    
+    
+    def __init__(self):
+        self.is_nested = None  # determines whether the this context is nested
+    
+    
+    def __enter__(self):
+        self.is_nested = nested_plotting_check._is_plotting
+        nested_plotting_check._is_plotting = True
+        return not self.is_nested
+    
+    def __exit__(self, *exc):
+        if not self.is_nested:
+            nested_plotting_check._is_plotting = False
 
 
 
@@ -95,6 +90,11 @@ def finalize_plot(fig_or_ax=None,
     Returns:
         tuple: The figure and the axes that were used to finalize the plot
     """
+    # Deprecated this method on 2020-06-13
+    warnings.warn("finalize_plot() method is deprecated. Use the decorators "
+                  "plot_on_axes or plot_on_figure to control plots instead.",
+                  DeprecationWarning)
+    
     # determine which figure to modify    
     if fig_or_ax is None:
         fig = plt.gcf()  # current figure
@@ -139,216 +139,6 @@ def disable_interactive():
 
 
 
-class plot_on_axes:
-    """ wrapper for a plot method that fills an axes
-    
-    Example:
-        The following example illustrates how the classes in this module can be
-        used to implement plotting on a class. In particular, supplying the
-        `update_method` will allow efficient dynamical plotting::
-    
-            class State:
-                def __init__(self):
-                    self.data = np.arange(8)
-    
-                def _update_plot(self, reference):
-                    reference.element.set_ydata(self.data)
-    
-                @plot_on_axes(update_method='_update_plot')
-                def plot(self, ax):
-                    line, = ax.plot(np.arange(8), self.data)
-                    return PlotReference(ax, line)
-    """
-    
-    
-    def __init__(self, update_method=None):
-        """
-        Args:
-            update_method (callable):
-                Method to call to update the plot. The argument of the new
-                method will be the result of the initial call of the wrapped
-                method.
-        """
-        # initialize the decorator
-        if callable(update_method):
-            raise RuntimeError('Wrapper needs to be called with brackets')
-        self.update_method = update_method
-        
-        
-    def __call__(self, method):
-        """ apply the actual decorator """
-        
-        def wrapper(obj, *,
-                    title: str = None,
-                    filename: str = None,
-                    show: bool = None,
-                    close_figure: bool = False,
-                    ax=None,
-                    **kwargs):
-            """
-            Args:
-                title (str):
-                    Title of the plot. If omitted, the title might be chosen
-                    automatically.
-                filename (str, optional):
-                    If given, the plot is written to the specified file.
-                show (bool):
-                    Flag setting whether :func:`matplotlib.pyplot.show` is
-                    called. The value `None` sets show to `True` by default, but
-                    disables it for nested calls.                    
-                close_figure (bool):
-                    Flag setting whether the figure is closed (after it was
-                    shown)
-            """
-            # some logic to check for nested plotting calls:
-            with nested_plotting_check() as is_outermost_plot_call:
-            
-                # disable interactive plotting temporarily
-                with disable_interactive():
-                    
-                    if ax is None:
-                        # create new figure
-                        backend = mpl.get_backend()
-                        if 'backend_inline' in backend or 'nbAgg' == backend:
-                            plt.close('all')  # close left over figures
-                        fig, ax = plt.subplots()
-                    else:
-                        fig = ax.get_figure()
-                
-                    # call the actual plotting function
-                    reference = method(obj, ax=ax, **kwargs)
-                    
-                    # finishing touches...            
-                    if title is not None:
-                        reference.ax.set_title(title)
-                    if filename:
-                        fig.savefig(filename)
-                        
-                # decide whether to show the final plot
-                if show or (show is None and is_outermost_plot_call):
-                    with warnings.catch_warnings():
-                        warnings.simplefilter("ignore")
-                        plt.show()
-                if close_figure:
-                    plt.close(fig)
-                
-            return reference
-            
-        wrapper.__name__ = method.__name__        
-        wrapper.__module__ = method.__module__
-        wrapper.__dict__.update(method.__dict__)
-        
-        if method.__doc__:
-            doc_fragment = wrapper.__doc__.splitlines()[:-1]
-            doc_fragment = '\n'.join(line[4:] for line in doc_fragment)
-            wrapper.__doc__ = re.sub(r'Args:\s*$', doc_fragment, method.__doc__,
-                                     flags=re.MULTILINE)
-
-        wrapper.mpl_class = 'axes'
-        wrapper.update_method = self.update_method
-            
-        return wrapper
-    
-
-
-class plot_on_figure:
-    """ wrapper for a plot method that fills an entire figure """
-    
-    
-    def __init__(self, update_method=None):
-        """
-        Args:
-            update_method (callable):
-                Method to call to update the plot. The argument of the new
-                method will be the result of the initial call of the wrapped
-                method.
-        """
-        # initialize the decorator
-        if callable(update_method):
-            raise RuntimeError('Wrapper needs to be called with brackets')
-        self.update_method = update_method
-        
-        
-    def __call__(self, method):
-        """ apply the actual decorator """
-        
-        def wrapper(obj, *,
-                    title: str = None,
-                    constrained_layout: bool = True,
-                    filename: str = None,
-                    show: bool = None,
-                    close_figure: bool = False,
-                    fig=None,
-                    **kwargs):
-            """
-            Args:
-                title (str):
-                    Title of the plot. If omitted, the title might be chosen
-                    automatically. This is shown above all panels.
-                constrained_layout (bool):
-                    Whether to use `constrained_layout` in 
-                    :func:`matplotlib.pyplot.figure` call to create a figure. 
-                    This affects the layout of all plot elements.
-                filename (str, optional):
-                    If given, the figure is written to the specified file.
-                show (bool):
-                    Flag setting whether :func:`matplotlib.pyplot.show` is
-                    called. The value `None` sets `show` to `True` by default,
-                    but disables it for nested calls.                  
-                close_figure (bool):
-                    Flag setting whether the figure is closed (after it was
-                    shown).
-            """
-            # some logic to check for nested plotting calls:
-            with nested_plotting_check() as is_outermost_plot_call:
-                
-                # disable interactive plotting temporarily
-                with disable_interactive():
-                    
-                    if fig is None:
-                        # create new figure
-                        backend = mpl.get_backend()
-                        if 'backend_inline' in backend or 'nbAgg' == backend:
-                            plt.close('all')  # close left over figures
-                        fig = plt.figure(constrained_layout=constrained_layout)
-                
-                    # call the actual plotting function
-                    reference = method(obj, fig=fig, **kwargs)
-                    
-                    # finishing touches...            
-                    if title is not None:
-                        fig.suptitle(title)
-                    if filename:
-                        fig.savefig(filename)
-                        
-                # decide whether to show the final plot
-                if show or (show is None and is_outermost_plot_call):
-                    with warnings.catch_warnings():
-                        warnings.simplefilter("ignore")
-                        plt.show()
-                        
-                if close_figure:
-                    plt.close(fig)
-                
-            return reference    
-            
-        wrapper.__name__ = method.__name__        
-        wrapper.__module__ = method.__module__
-        wrapper.__dict__.update(method.__dict__)
-        
-        if method.__doc__:
-            doc_fragment = wrapper.__doc__.splitlines()[:-1]
-            doc_fragment = '\n'.join(line[4:] for line in doc_fragment)
-            wrapper.__doc__ = re.sub(r'Args:\s*$', doc_fragment, method.__doc__,
-                                     flags=re.MULTILINE)
-
-        wrapper.mpl_class = 'figure'
-        wrapper.update_method = self.update_method
-        
-        return wrapper
-
-
-
 class PlotReference():
     """ contains all information to update a plot element """
     
@@ -367,6 +157,318 @@ class PlotReference():
 
 
 
+def plot_on_axes(wrapped=None, update_method=None):
+    """ decorator for a plot method or function that uses a single axes
+    
+    This decorator adds typical options for creating plots that fill a single
+    axes. These options are available via keyword arguments. These options can
+    be described in the docstring, if the placeholder `{PLOT_ARGS}` is mentioned
+    in the docstring of the wrapped function or method. Note that the decorator
+    can be used on both functions and methods.
+
+    Example:
+        The following example illustrates how this decorator can be used to
+        implement plotting for a given class. In particular, supplying the
+        `update_method` will allow efficient dynamical plotting::
+    
+            class State:
+                def __init__(self):
+                    self.data = np.arange(8)
+    
+                def _update_plot(self, reference):
+                    reference.element.set_ydata(self.data)
+    
+                @plot_on_axes(update_method='_update_plot')
+                def plot(self, ax):
+                    line, = ax.plot(np.arange(8), self.data)
+                    return PlotReference(ax, line)
+                    
+                    
+            @plot_on_axes
+            def make_plot(ax):
+                ax.plot(...)
+                    
+        When `update_method` is not supplied, the method can still be used for
+        plotting, but dynamic updating, e.g., by
+        :class:`pde.trackers.PlotTracker`, is not possible.
+
+    Args:
+        wrapped (callable):
+            Function to be wrapped
+        update_method (callable or str):
+            Method to call to update the plot. The argument of the new
+            method will be the result of the initial call of the wrapped
+            method.
+    """
+    if wrapped is None:
+        # handle the case where decorator was called without brackets
+        return functools.partial(plot_on_axes, update_method=update_method)    
+    
+    def wrapper(*args,
+                title: str = None,
+                filename: str = None,
+                action: str = 'auto',
+                ax_style: Dict[str, Any] = None,
+                ax=None,
+                **kwargs):
+        """
+        title (str):
+            Title of the plot. If omitted, the title might be chosen
+            automatically.
+        filename (str, optional):
+            If given, the plot is written to the specified file.
+        action (str):
+            Decides what to do with the figure. If the argument is set to `show`
+            :func:`matplotlib.pyplot.show` will be called to show the plot, if
+            the value is `create`, the figure will be created, but not shown,
+            and the value `close` closes the figure, after saving it to a file 
+            when `filename` is given. The default value `auto` implies that the
+            plot is shown if it is not a nested plot call.
+        ax_style (dict):
+            Dictionary with properties that will be changed on the axis
+            after the plot has been drawn by calling
+            :meth:`matplotlib.pyplot.setp`.
+        ax (:class:`matplotlib.axes.Axes`):
+            Figure axes to be used for plotting. If `None`, a new figure
+            is created. This has no effect if a `reference` is supplied.
+        """
+        # Note on docstring: This docstring replaces the token {PLOT_ARGS} in 
+        # the wrapped function
+        
+        # some logic to check for nested plotting calls:
+        with nested_plotting_check() as is_outermost_plot_call:
+        
+            # disable interactive plotting temporarily
+            with disable_interactive():
+                
+                if ax is None:
+                    # create new figure
+                    backend = mpl.get_backend()
+                    if 'backend_inline' in backend or 'nbAgg' == backend:
+                        plt.close('all')  # close left over figures
+                    fig, ax = plt.subplots()
+                else:
+                    fig = ax.get_figure()
+            
+                # call the actual plotting function
+                reference = wrapped(*args, ax=ax, **kwargs)
+                
+                # finishing touches...            
+                if title is not None:
+                    ax.set_title(title)
+                if ax_style:
+                    plt.setp(ax, **ax_style)
+                if filename:
+                    fig.savefig(filename)
+                    
+            # decide what to do with the final plot
+            if action == 'auto':
+                if is_outermost_plot_call:
+                    action = 'show'
+                else:
+                    action = 'create'
+                    
+            if action == 'show':
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore")
+                    plt.show()
+                    
+            elif action == 'close':
+                plt.close(fig)
+                
+            elif action != 'create':
+                raise ValueError(f'Unknown action `{action}`')
+            
+        return reference
+        
+    # adjusting the signature of the wrapped function to include wrapper args
+    import inspect
+    sig_wrapped = inspect.signature(wrapped)
+    parameters = tuple(arg
+                       for name, arg in sig_wrapped.parameters.items()
+                       if name != 'kwargs' and name != 'ax')
+    
+    sig_wrapper = inspect.signature(wrapper)
+    parameters += tuple(sig_wrapper.parameters.values())
+    wrapper.__signature__ = sig_wrapped.replace(parameters=parameters)
+    
+    # adjusting additional properties of the function to match the wrapped one
+    wrapper.__name__ = wrapped.__name__        
+    wrapper.__module__ = wrapped.__module__
+    wrapper.__dict__.update(wrapped.__dict__)
+    
+    if wrapped.__doc__:
+        replace_in_docstring(wrapper, '{PLOT_ARGS}', wrapper.__doc__,
+                             docstring=wrapped.__doc__)
+
+    wrapper.mpl_class = 'axes'
+    wrapper.update_method = update_method
+        
+    return wrapper
+    
+
+
+def plot_on_figure(wrapped=None, update_method=None):
+    """ decorator for a plot method or function that fills an entire figure
+    
+    This decorator adds typical options for creating plots that fill an
+    entire figure. These options are available via keyword arguments. These
+    options can be described in the docstring, if the placeholder `{PLOT_ARGS}`
+    is mentioned in the docstring of the wrapped function or method. Note that
+    the decorator can be used on both functions and methods.
+    
+    Example:
+        The following example illustrates how this decorator can be used to
+        implement plotting for a given class. In particular, supplying the
+        `update_method` will allow efficient dynamical plotting::
+    
+            class State:
+                def __init__(self):
+                    self.data = np.random.random((2, 8))
+    
+                def _update_plot(self, reference):
+                    ref1, ref2 = reference
+                    ref1.element.set_ydata(self.data[0])
+                    ref2.element.set_ydata(self.data[1])
+    
+                @plot_on_figure(update_method='_update_plot')
+                def plot(self, fig):
+                    ax1, ax2 = fig.subplots(1, 2)
+                    l1, = ax1.plot(np.arange(8), self.data[0])
+                    l2, = ax2.plot(np.arange(8), self.data[1])
+                    return [PlotReference(ax1, l1), PlotReference(ax2, l2)]
+                   
+                    
+            @plot_on_figure
+            def make_plot(fig):
+                ...
+                    
+                    
+        When `update_method` is not supplied, the method can still be used for
+        plotting, but dynamic updating, e.g., by
+        :class:`pde.trackers.PlotTracker`, is not possible. 
+        
+        
+    Args:
+        wrapped (callable):
+            Function to be wrapped
+        update_method (callable or str):
+            Method to call to update the plot. The argument of the new
+            method will be the result of the initial call of the wrapped
+            method.
+    """
+    if wrapped is None:
+        # handle the case where decorator was called without brackets
+        return functools.partial(plot_on_figure, update_method=update_method)    
+    
+    
+    def wrapper(*args, **kwargs):
+        """
+        title (str):
+            Title of the plot. If omitted, the title might be chosen
+            automatically. This is shown above all panels.
+        constrained_layout (bool):
+            Whether to use `constrained_layout` in 
+            :func:`matplotlib.pyplot.figure` call to create a figure. 
+            This affects the layout of all plot elements.
+        filename (str, optional):
+            If given, the figure is written to the specified file.
+        action (str):
+            Decides what to do with the figure. If the argument is set to `show`
+            :func:`matplotlib.pyplot.show` will be called to show the plot, if
+            the value is `create`, the figure will be created, but not shown,
+            and the value `close` closes the figure, after saving it to a file 
+            when `filename` is given. The default value `auto` implies that the
+            plot is shown if it is not a nested plot call.
+        fig_style (dict):
+            Dictionary with properties that will be changed on the
+            figure after the plot has been drawn by calling
+            :meth:`matplotlib.pyplot.setp`.
+        fig (:class:`matplotlib.figures.Figure`):
+            Figure that is used for plotting. If omitted, a new figure is
+            created.
+        """
+        # Note on docstring: This docstring replaces the token {PLOT_ARGS} in 
+        # the wrapped function
+        
+        title = kwargs.pop('title', None)
+        constrained_layout = kwargs.pop('constrained_layout', True)
+        filename = kwargs.pop('filename', None)
+        action = kwargs.pop('action', 'auto')
+        fig_style = kwargs.pop('fig_style', {})
+        fig = kwargs.pop('fig', None)
+        
+        # some logic to check for nested plotting calls:
+        with nested_plotting_check() as is_outermost_plot_call:
+            
+            # disable interactive plotting temporarily
+            with disable_interactive():
+                
+                if fig is None:
+                    # create new figure
+                    backend = mpl.get_backend()
+                    if 'backend_inline' in backend or 'nbAgg' == backend:
+                        plt.close('all')  # close left over figures
+                    fig = plt.figure(constrained_layout=constrained_layout)
+            
+                # call the actual plotting function
+                reference = wrapped(*args, fig=fig, **kwargs)
+                
+                # finishing touches...            
+                if title is not None:
+                    fig.suptitle(title)
+                if fig_style:
+                    plt.setp(fig, **fig_style)
+                if filename:
+                    fig.savefig(filename)
+                    
+            # decide what to do with the final plot
+            if action == 'auto':
+                if is_outermost_plot_call:
+                    action = 'show'
+                else:
+                    action = 'create'
+                    
+            if action == 'show':
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore")
+                    plt.show()
+                    
+            elif action == 'close':
+                plt.close(fig)
+                
+            elif action != 'create':
+                raise ValueError(f'Unknown action `{action}`')  
+                      
+        return reference    
+        
+    # adjusting the signature of the wrapped function to include wrapper args
+    import inspect
+    sig_wrapped = inspect.signature(wrapped)
+    parameters = tuple(arg
+                       for name, arg in sig_wrapped.parameters.items()
+                       if name != 'kwargs' and name != 'fig')
+    
+    sig_wrapper = inspect.signature(wrapper)
+    parameters += tuple(sig_wrapper.parameters.values())
+    wrapper.__signature__ = sig_wrapped.replace(parameters=parameters)
+    
+    # adjusting additional properties of the function to match the wrapped one
+    wrapper.__name__ = wrapped.__name__        
+    wrapper.__module__ = wrapped.__module__
+    wrapper.__dict__.update(wrapped.__dict__)
+    
+    if wrapped.__doc__:
+        replace_in_docstring(wrapper, '{PLOT_ARGS}', wrapper.__doc__,
+                             docstring=wrapped.__doc__)
+
+    wrapper.mpl_class = 'figure'
+    wrapper.update_method = update_method
+    
+    return wrapper
+
+
+
 class PlottingContextBase(object):
     """ base class of the plotting contexts
     
