@@ -7,7 +7,7 @@ Functions and classes for plotting simulation data
    ScalarFieldPlot
    plot_magnitudes
    plot_kymograph
-   
+   plot_kymographs
    
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 '''
@@ -25,9 +25,10 @@ from ..fields.base import FieldBase, DataFieldBase
 from ..storage.base import StorageBase
 from ..tools.misc import display_progress
 from ..tools.docstrings import fill_in_docstring
-from ..tools.plotting import finalize_plot
+from ..tools.plotting import PlotReference, plot_on_axes, plot_on_figure
 
 
+_logger = logging.getLogger(__name__)
 ScaleData = Union[str, float, Tuple[float, float]]
 
 
@@ -190,8 +191,12 @@ class ScalarFieldPlot():
         Returns:
             :class:`~pde.visualization.plotting.ScalarFieldPlot`
         """
-        fields = storage.get_field(0)
-        quantities = cls._prepare_quantities(fields, quantities=quantities,
+        fields = storage[0]
+        assert isinstance(fields, FieldBase)
+        
+        # prepare the data that needs to be plotted
+        quantities = cls._prepare_quantities(fields,
+                                             quantities=quantities,
                                              scale=scale)
         
         # resolve automatic scaling
@@ -477,39 +482,30 @@ class ScalarFieldPlot():
 
 
 
+@plot_on_axes()
 @fill_in_docstring
 def plot_magnitudes(storage: StorageBase,
-                    filename: str = None,
                     quantities=None,
                     ax=None,
-                    title: str = None,
-                    show: bool = False,
-                    **kwargs) -> None:
-    r""" create a plot of spatially integrated quantities from a given storage.
-    
-    Note that the plot will not be displayed when a filename is given and no
-    plot axes are specified.
+                    **kwargs) -> PlotReference:
+    r""" plot spatially integrated quantities as a function of time
     
     Args:
         storage:
             Instance of :class:`~pde.storage.base.StorageBase` that contains
             the simulation data that will be plotted
-        filename (str):
-            If given, the resulting image is written to this file.
         quantities:
             {ARG_PLOT_QUANTITIES}
-        ax:
-            Matplotlib figure axes to be used for plotting. If `None`, a new
-            figure is created
-        title (str):
-            Determines the title of the figure
-        show (bool):
-            Flag determining whether :func:`matplotlib.pyplot.show` is called
+        {PLOT_ARGS}
         \**kwargs:
             All remaining parameters are forwarded to the `ax.plot` method
+            
+    Returns:
+        :class:`~pde.tools.plotting.PlotReference`: The reference to the plot
     """
     if quantities is None:
-        fields = storage.get_field(0)
+        fields = storage[0]
+        assert isinstance(fields, FieldBase)
         if fields.label:
             label_base = fields.label 
         else:
@@ -545,15 +541,12 @@ def plot_magnitudes(storage: StorageBase,
             value *= quantity.get('scale', 1)
             data[i]['values'].append(value)
     
-    close_figure = (bool(filename) and ax is None)
-    if ax is None:
-        # create new figure
-        ax = plt.figure().gca()
-    
     # plot the data
+    lines = []
     for d in data:
         kwargs['label'] = d['label']
-        ax.plot(storage.times, d['values'], **kwargs)
+        l, = ax.plot(storage.times, d['values'], **kwargs)
+        lines.append(l)
         
     ax.set_xlabel('Time')
     if len(data) == 1:
@@ -561,36 +554,82 @@ def plot_magnitudes(storage: StorageBase,
         
     if len(data) > 1:
         ax.legend(loc='best')
-    
-    finalize_plot(ax, title=title, filename=filename, show=show,
-                  close_figure=close_figure)
+        
+    return PlotReference(ax, lines)
             
 
 
+def _plot_kymograph(img_data: Dict[str, Any],
+                    ax,
+                    colorbar: bool = True,
+                    transpose: bool = False,
+                    **kwargs) -> PlotReference:
+    r""" plots a simple kymograph from given data
+    
+    Args:
+        img_data (dict):
+            Contains the kymograph data
+        ax (:class:`~matplotlib.axes.Axes`):
+            The axes to which the plot is added
+        colorbar (bool):
+            Whether to show a colorbar or not
+        transpose (bool):
+            Determines whether the transpose of the data should is plotted
+        \**kwargs:
+            Additional keyword arguments are passed to
+            :func:`matplotlib.pyplot.imshow`.
+            
+    Returns:
+        :class:`~pde.tools.plotting.PlotReference`: The reference to the plot
+    """
+    # transpose data if requested
+    if transpose:
+        # avoid changing the img_data that was passed into the function
+        extent = np.r_[img_data['extent_y'], img_data['extent_x']]
+        img_data = {'data': img_data['data'].T,
+                    'label_x': img_data['label_y'],
+                    'label_y': img_data['label_x']}
+    else:
+        extent = np.r_[img_data['extent_x'], img_data['extent_y']]
+    
+    # create the actual plot
+    ref = ax.imshow(img_data['data'], extent=extent, origin='lower', **kwargs)
+    
+    # adjust some settings
+    ax.set_xlabel(img_data['label_x'])
+    ax.set_ylabel(img_data['label_y'])
+    ax.set_aspect('auto') 
+
+    if colorbar:
+        from ..tools.misc import add_scaled_colorbar
+        add_scaled_colorbar(ref, ax=ax)
+        
+    return PlotReference(ax, ref)
+
+
+
+@plot_on_axes()
 def plot_kymograph(storage: StorageBase,
-                   filename: str = None,
+                   field_index: int = None,
                    scalar: str = 'auto',
                    extract: str = 'auto',
                    colorbar: bool = True,
                    transpose: bool = False,
-                   title: str = None,
-                   show: bool = False,
                    ax=None,
-                   **kwargs):
-    r""" plots a simple kymograph from stored data
+                   **kwargs) -> PlotReference:
+    r""" plots a single kymograph from stored data
     
     The kymograph shows line data stacked along time. Consequently, the
     resulting image shows space along the horizontal axis and time along the
     vertical axis.
     
-    Note that the plot will not be displayed when a filename is given and no
-    plot axes are specified.
-    
     Args:
         storage (:class:`~droplets.simulation.storage.StorageBase`):
             The storage instance that contains all the data for the movie
-        filename (str):
-            If given, the resulting image is written to this file.
+        field_index (int):
+            An index to choose a single field out of many in a collection
+            stored in `storage`. This option should not be used if only a single
+            field is stored in a collection.
         scalar (str):
             The method for extracting scalars as described in
             :meth:`DataFieldBase.to_scalar`.
@@ -601,53 +640,119 @@ def plot_kymograph(storage: StorageBase,
             Whether to show a colorbar or not
         transpose (bool):
             Determines whether the transpose of the data should is plotted
-        title (str):
-            Determines the title of the figure
-        show (bool):
-            Flag determining whether :func:`matplotlib.pyplot.show` is called
-        ax: Figure axes to be used for plotting. If `None`, a new figure is
-            created
+        {PLOT_ARGS}
         \**kwargs:
             Additional keyword arguments are passed to
             :func:`matplotlib.pyplot.imshow`.
             
     Returns:
-        Result of :func:`matplotlib.pyplot.imshow`
+        :class:`~pde.tools.plotting.PlotReference`: The reference to the plot
     """
-    close_figure = (bool(filename) and ax is None)
-    if ax is None:
-        ax = plt.figure().gca()  # create new figure
-
-    full_data = []
-    for _, data in storage.items():
-        img_data = data.get_line_data(scalar=scalar, extract=extract)
-        full_data.append(img_data['data_y'])
-        
-    full_data = np.array(full_data)
-    extent = np.r_[img_data['extent_x'], storage.times[0], storage.times[-1]]
+    if len(storage) == 0:
+        raise RuntimeError('Storage is empty')
+    line_data_args: Dict[str, Any] = {'scalar': scalar, 'extract': extract}
     
-    if transpose:
-        full_data = full_data.T  # type: ignore
-        label_x, label_y = 'Time', img_data['label_x']
+    if field_index is not None:
+        if storage.has_collection:
+            line_data_args['index'] = field_index
+        else:
+            _logger.warning('`field_index` should only be set for '
+                            'FieldCollections')
+    
+    # prepare the image data for one kymograph
+    image = []
+    for _, field in storage.items():
+        img_data = field.get_line_data(**line_data_args)
+        image.append(img_data['data_y'])
+        
+    img_data['data'] = np.array(image)
+    img_data['extent_y'] = (storage.times[0], storage.times[-1])
+    img_data['label_y'] = 'Time'
+    
+    return _plot_kymograph(img_data, ax, colorbar=colorbar, transpose=transpose,
+                           **kwargs)
+    
+    
+
+@plot_on_figure()
+def plot_kymographs(storage: StorageBase,
+                    scalar: str = 'auto',
+                    extract: str = 'auto',
+                    colorbar: bool = True,
+                    transpose: bool = False,
+                    resize_fig: bool = True,
+                    fig=None,
+                    **kwargs) -> List[PlotReference]:
+    r""" plots kymographs for all fields stored in `storage`
+    
+    The kymograph shows line data stacked along time. Consequently, the
+    resulting image shows space along the horizontal axis and time along the
+    vertical axis.
+    
+    Args:
+        storage (:class:`~droplets.simulation.storage.StorageBase`):
+            The storage instance that contains all the data for the movie
+        scalar (str):
+            The method for extracting scalars as described in
+            :meth:`DataFieldBase.to_scalar`.
+        extract (str):
+            The method used for extracting the line data. See the docstring
+            of the grid method `get_line_data` to find supported values.
+        colorbar (bool):
+            Whether to show a colorbar or not
+        transpose (bool):
+            Determines whether the transpose of the data should is plotted
+        resize_fig (bool):
+            Whether to resize the figure to adjust to the number of panels
+        {PLOT_ARGS}
+        \**kwargs:
+            Additional keyword arguments are passed to the calls to
+            :func:`matplotlib.pyplot.imshow`.
+            
+    Returns:
+        list of :class:`~pde.tools.plotting.PlotReference`: The references to
+        all plots
+    """
+    if len(storage) == 0:
+        raise RuntimeError('Storage is empty')
+    line_data_args = {'scalar': scalar, 'extract': extract}
+    if storage.has_collection:
+        num_fields = len(storage[0])  # type: ignore
     else:
-        label_x, label_y = img_data['label_x'], 'Time'
+        num_fields = 1
     
-    res = ax.imshow(full_data, extent=extent, origin='lower', **kwargs)
+    # prepare the image data for one kymograph
+    images = []
+    for _, field in storage.items():
+        if storage.has_collection:
+            image_lines = []
+            for f_id in range(num_fields):
+                img_data = field.get_line_data(index=f_id,  # type: ignore
+                                               **line_data_args)
+                image_lines.append(img_data['data_y'])
+        else:
+            img_data = field.get_line_data(**line_data_args)
+            image_lines = [img_data['data_y']]
+        images.append(image_lines)
     
-    ax.set_xlabel(label_x)
-    ax.set_ylabel(label_y)
-    ax.set_aspect('auto') 
-
-    if colorbar:
-        from ..tools.misc import add_scaled_colorbar
-        add_scaled_colorbar(res, ax=ax)
-        
-    if title is None:
-        title = img_data['label_y']
-
-    finalize_plot(ax, title=title, filename=filename, show=show,
-                  close_figure=close_figure)
+    img_arr = np.array(images)
+    img_data['extent_y'] = (storage.times[0], storage.times[-1])
+    img_data['label_y'] = 'Time'
     
-    return res
+    # disable interactive plotting temporarily
+    # create a plot with all the panels 
+    if resize_fig:
+        fig.set_size_inches((4 * num_fields, 3), forward=True)
+    axs = fig.subplots(1, num_fields, squeeze=False)
+    
+    # iterate over all axes and plot the kymograph
+    refs = []
+    for i, ax in enumerate(axs[0]):
+        img_data['data'] = img_arr[:, i]
+        ref = _plot_kymograph(img_data, ax, colorbar=colorbar,
+                              transpose=transpose, **kwargs)
+        if storage.has_collection:
+            ax.set_title(storage._field[i].label)  # type: ignore
+        refs.append(ref)
 
-    
\ No newline at end of file
+    return refs
